#include "utils/cmd2argv.h"
#include "utils/utils.h"
#include "external.h"

static void sig_pipe(int signo) {
	warn("SIGPIPE was received");
}


char *run_external_program(const char *cmd_line, const char *to_stdin) {
	int n, fd1[2], fd2[2];
	pid_t pid;
	char *from_stdout = NULL;
	if ((from_stdout = (char *)malloc(MAXLINE)) == NULL) {
		warn("Can't allocate memory for from_stdout");
		return NULL;
	}

	if (signal(SIGPIPE, sig_pipe) == SIG_ERR)
		warn("error signal function");
	if (pipe(fd1) < 0) {
		warn("error pipe function");
		free(from_stdout);
		return NULL;
	} else if (pipe(fd2) < 0) {
		warn("error pipe function");
		close(fd1[0]);
		close(fd1[1]);
		free(from_stdout);
		return NULL;
	}
	if ((pid = fork()) < 0) {
		warn("error fork function");
		free(from_stdout);
		return NULL;
	} else if (pid > 0) {
		close(fd1[0]);
		close(fd2[1]);

		n = strlen(to_stdin);
		if (write(fd1[1], to_stdin, n) != n)
			warn("error output to channel");
		if ((n = read(fd2[0], from_stdout, MAXLINE)) < 0) {
			warn("error read from channel");
			return NULL;
		}
		from_stdout[n] = 0;
		debug("get from external auth prog: |%s|", from_stdout);

		if (ferror(stdin))
			warn("error read from stdin");
	} else {
		close(fd1[1]);
		close(fd2[0]);
		if (fd1[0] != STDIN_FILENO) {
			if (dup2(fd1[0], STDIN_FILENO) != STDIN_FILENO)
				warn("error call dup2 for stdin");
			close(fd1[0]);
		}
		if (fd2[1] != STDOUT_FILENO) {
			if (dup2(fd2[1], STDOUT_FILENO) != STDOUT_FILENO)
				warn("error call dup2 for stdout");
			close(fd2[1]);
		}
		char **argv = cmd2argv(cmd_line);
		if (argv == NULL) {
			err("cmd2argv return NULL");
			free(from_stdout);
			return NULL;
		}
		if (execv(argv[0], argv) < 0) {
			warn("error call execv");
			free(stdout);
			return NULL;
		}
		/* TODO: free argv */
	}

	return from_stdout;
}
