#define _GNU_SOURCE
#include <stdio.h>
#include <unistd.h>
#include <dlfcn.h>

#include "def.h"
#include "utils/utils.h"
#include "conf/conf.h"
#include "module/module_api.h"


int (*type_auth_client)(char *login, char *password);
char *(*type_get_client_password)(char *login);

// Function for auth client, which check login&password in auth modules and db, get permission from bz module.
int auth_client(char *login, char *password, char *type_service) {
	debug("auth_client");
	int checklp=0;

	kcfg *cfg = get_katrin_cfg();
	if (cfg == NULL) {
		return 0;
	}

	/* Load libkatrin-bz-<type_service> */
	bz_module *bz = get_bz_module(type_service);
	if (bz == NULL) {
		debug("Can't get %s bz module. Load..", type_service);
		if (add_module(BZ, type_service) != 0)
			err("Error load bz module");
		bz = get_bz_module(type_service);
	}


	debug("call allow_auth(%s)", login);

	debug("bz allow_auth returned %d", bz->bz_api.allow_auth(login));
	if (bz->bz_api.allow_auth(login)==1) {
		debug("ok allowauth");
		list *p;
		for (p = cfg->types_auth; p != NULL; p = p->next) {

			debug("type auth lib: %s", p->value);

			int ret;
			void *handleLibTypeAuth;

			/* Load lib for auth */
			ret = load_library("auth", p->value, &handleLibTypeAuth);
			if (ret) {
				free_katrin_cfg(cfg);
				return 0;
			}

			type_auth_client = dlsym(handleLibTypeAuth, "type_auth_client");
			if (type_auth_client(login, password) == 1) {
				checklp = 1;
			}

			dlclose(handleLibTypeAuth);

			if (checklp==1) {
				free_katrin_cfg(cfg);
				return 1;
			}
		}
	}
	else {
		debug("allow_auth(%s) doesn't allow auth client with this login", login);
	}
	free_katrin_cfg(cfg);
	return 0;
}

char *get_client_password(char *login) {
	char *password = NULL;

	kcfg *cfg = get_katrin_cfg();
	if (cfg == NULL) {
		return NULL;
	}

	list *p;
	for (p = cfg->types_auth; p != NULL; p = p->next) {

		debug("type auth lib: %s", p->value);

		int ret;
		void *handleLibTypeAuth;

		/* Load lib for auth */
		ret = load_library("auth", p->value, &handleLibTypeAuth);
		if (ret) {
			free_katrin_cfg(cfg);
			return NULL;
		}

		type_get_client_password = dlsym(handleLibTypeAuth, "type_get_client_password");
		password = type_get_client_password(login);

		dlclose(handleLibTypeAuth);

		if (password != NULL) {
			free_katrin_cfg(cfg);
			return password;
		}

	}

	free_katrin_cfg(cfg);
	return password;

}
