#define _GNU_SOURCE
#include "bz/bz.h"

int allow_auth_default(char *login, char *serviceName) {
	debug("allow_auth");
	notice("allow_auth");
	notice("login: %s", login);

	kcfg *cfg = get_katrin_cfg();
	debug("Load db lib");
	add_module(DB, cfg->db_mod);
	free_katrin_cfg(cfg);

	struct client *client = NULL;
	struct serviceslist *sl = NULL;
	struct tariff tariff;
	struct account account;

	client = db.db_api.get_client(0, login, NULL, NULL, NULL);
	if (!client) {
		err("Client %s wasn't found, denying access",login);	
		return 0;
	}

	// First check services in accounts of client clientgroups
	sl = db.db_api.get_client_clientgroups_services(client->id, serviceName);
	if(!sl) 
		// Check if client has services in individual accounts
		sl = db.db_api.get_client_services(client->id, Personal, serviceName);
	if (!sl) {
		err("Client %s doesn't have service %s, denying access",login,serviceName);
		return 0;
	}

	tariff = sl->tariff;
	account = sl->account;
	free_serviceslist(sl);
	notice("balance: %1.2f", account.balance);
	notice("credit: %1.2f", account.credit);
	int rezult = 0;
	if (account.balance + account.credit > 0 && account.blocked == 0) {
		//Prepare vars
		time_t t;
		struct tm *tmp;

		t = time(NULL);
		tmp = localtime(&t);
		if (tmp == NULL)
			warn("Can't get local time");

		//The hour as a decimal number using a 24-hour clock (range 00 to 23).
		char hour[3];
		strftime(hour, sizeof(hour), "%H", tmp);
		debug("hour: %s",hour);
		//The day of the week as a decimal, range 1 to 7, Monday being 1.
		char week_day[2];
		strftime(week_day, sizeof(week_day), "%u", tmp);
		debug("week_day: %s",week_day);
		//The day of the month as a decimal number (range 01 to 31).
		char month_day[3];
		strftime(month_day, sizeof(month_day), "%d", tmp);
		debug("month_day: %s",month_day);
		//The month as a decimal number (range 01 to 12).
		char year_month[3];
		strftime(year_month, sizeof(year_month), "%m", tmp);
		debug("year_month: %s",year_month);

		debug("tariff.week_days: %s",tariff.week_days);

		if (strlen(tariff.week_days)==0 || strstr(tariff.week_days,week_day)!=0)
			rezult = 1;
	}
	free(client);

	return rezult;

}


