#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "bz/bz.h"
#include "stat/libkatrin-stat-traff.h"
#include "utils/traff.h"

struct client_traff_info *service2clientService(struct traff_info *traff) {
	debug("traffInfo2clientTraffInfo");
	debug("Traff info struct:");
	debug("\ttraff->src_port = %d", traff->src_port);
	debug("\ttraff->proto = %d", traff->proto);
	debug("\ttraff->bytes = %d", traff->bytes);

	char *s1;
	char *s2;

	s1 = strdup(inet_ntoa(traff->src_addr));
	s2 = strdup(inet_ntoa(traff->dst_addr));

	debug("\ttraff->src_addr = %s \n\ttraff->dst_addr = %s ", s1, s2);

	free(s1);
	free(s2);


	struct client_traff_info *pUTI = (struct client_traff_info *) malloc(sizeof(struct client_traff_info));
	pUTI->traffic = (*traff);
	//	Get info
	int tariffid = 0;
	int accountid = 0;
	int inputtraff = 0;
	int outputtraff = 0;
	// Init stub pUTI
	pUTI->client_id = 0;
	pUTI->account_id = 0;
	pUTI->filterid = 0;
	pUTI->cost = 0;

	struct client *client = db.db_api.get_client(0, NULL, "traff", "ip", inet_ntoa(traff->dst_addr));
	if (client == NULL) {
		client = db.db_api.get_client(0, NULL, "traff", "ip",  inet_ntoa(traff->src_addr));
		if (client != NULL) {
			outputtraff = 1;
			pUTI->in_traff = OUTPUT_TRAFF;
		}
	}
	else {
		inputtraff = 1;
		pUTI->in_traff = INPUT_TRAFF;
	}

	if (client != NULL ) {
		struct serviceslist *sl;
		// First check services in accounts of client clientgroups
		sl = db.db_api.get_client_clientgroups_services(client->id, "traff");
		if(!sl) 
			// Check if client has services in individual accounts
			sl = db.db_api.get_client_services(client->id, Personal, "traff");
		if(!sl) {
			// Client doesn't use any services at all
			err("Client %d doesn't have services",client->id);
			free_client(client);
			return pUTI;
		}
		else {
			tariffid = sl->tariff.id;
			accountid = sl->account.id;
			free(sl);
			pUTI->client_id = client->id;
			pUTI->account_id = accountid;
			free_client(client);
		}
	}
	else {
		debug("client is NULL");
		return pUTI;
	}

	//	Apply filters
	int usefilter = 0;
	int applyfilter = 0;
	int store_service_stat = 0;

	struct filterslist *filters = db.db_api.tariffid2filters(tariffid, "traff");
	struct filterslist *pfl = NULL;

	if (filters == NULL)
		return pUTI;

	//Prepare vars
	time_t t;
	struct tm *tmp;

	t = time(NULL);
	tmp = localtime(&t);
	if (tmp == NULL)
		warn("Can't get local time");

	//The hour as a decimal number using a 24-hour clock (range 00 to 23).
	char hour[3];
	strftime(hour, sizeof(hour), "%H", tmp);
	debug("hour: %s", hour);
	//The day of the week as a decimal, range 1 to 7, Monday being 1.
	char week_day[2];
	strftime(week_day, sizeof(week_day), "%u", tmp);
	debug("week_day: %s", week_day);
	//The day of the month as a decimal number (range 01 to 31).
	char month_day[3];
	strftime(month_day, sizeof(month_day), "%d", tmp);
	debug("month_day: %s", month_day);
	//The month as a decimal number (range 01 to 12).
	char year_month[3];
	strftime(year_month, sizeof(year_month), "%m", tmp);
	debug("year_month: %s", year_month);

	for (; filters!=NULL; filters=pfl ) {
		if (applyfilter == 1)
			goto next_and_free_filter;
		usefilter = 0;
//		struct in_addr addr_mask;
		struct in_addr addr_ip;
//		struct in_addr addr_net;

		int in_traff = atoi(get_param_value("in_traff", filters->filter.params));
		char *network_list_str = get_param_value("network_list", filters->filter.params);
		char *netmask = get_param_value("netmask", filters->filter.params);
		int port = atoi(get_param_value("port", filters->filter.params));
		char *day_hours = get_param_value("day_hours", filters->filter.params);
		char *week_days = get_param_value("week_days", filters->filter.params);
		char *month_days = get_param_value("month_days", filters->filter.params);
		char *year_months = get_param_value("year_months", filters->filter.params);
		double mb_cost = atof(get_param_value("mb_cost", filters->filter.params));
//		inet_aton(network, &addr_net);
//		inet_aton(netmask, &addr_mask);

//		debug("filters.network: %s filter.netmask: %s", network, netmask);

		if (inputtraff==1)
			addr_ip = traff->src_addr;
		if (outputtraff==1)
			addr_ip = traff->dst_addr;
		//Check of conditions
		debug("Check traffic direction...");
		if (inputtraff && in_traff || outputtraff && !in_traff) {

			debug("Check net area...");
//			if (((addr_ip.s_addr & addr_mask.s_addr) == addr_net.s_addr) || (strcmp(network,"")==0 || strcmp(netmask,"")==0) ) {
			// FIXME: Optimize over cache network list struct for each filter.
			struct network_list *nl = get_network_list(network_list_str);
			int r = check_ip_in_list(htonl(addr_ip.s_addr), nl);
			free_network_list(nl);
			if (1 == r) {
//				debug("IP inside net area: %s %s",network, netmask);
				// Check port
				if (port==-1 || (port==traff->src_port && inputtraff==1) || (port==traff->dst_port && outputtraff==1)) {
					debug("Apply port filter");
					debug("filters.port=%d srcport=%d dstport=%d inputtraff=%d outputtraff=%d", 
									port, 
									traff->src_port, 
									traff->dst_port, 
									inputtraff, 
									outputtraff);
					//Check day_hours
					debug("Check day_hours...");
					if (strcmp(day_hours, "")==0 || strstr(day_hours, hour)!=NULL) {
						debug("Hour inside day_hours: %s in %s", hour, day_hours);
						if (strcmp(week_days, "")==0 || strstr(week_days, week_day)!=NULL) {
							debug("Week_day inside : %s in %s",week_day, week_days);
							if (strcmp(month_days, "")==0 || strstr(month_days, month_day)!=NULL) {
								debug("Month_day inside : %s in %s", month_day, month_days);
								if (strcmp(year_months, "")==0 || strstr(year_months, year_month)!=NULL) {
									debug("Year_month inside : %s in %s", year_month, year_months);
								// If _all_ check true, then for this traff use filter
								usefilter=1;
								debug("usefilter set to 1");
								store_service_stat = filters->filter.store_stat;
								}
							}
						}
					}
				}
			}
		}
		//Apply cost from first checked filter and exit from check filters ELSE check next filter/
		if (usefilter == 1) {
			if (0 != mb_cost)
				pUTI->cost = ((double)traff->bytes/1048576.0)*mb_cost;
			pUTI->filterid = filters->filter.id;
			applyfilter = 1;
		}
next_and_free_filter:
		pfl = filters->next;
		free_paramlist(filters->filter.params);
		free(filters);
	}

	if (applyfilter == 0) {
		pUTI->cost = 0;
		pUTI->filterid = 0;
	}

	// Store stat before return result
	if (usefilter == 1 && store_service_stat == 1)
		store_stat(pUTI);

	debug("exit from traffInfo2clientTraffInfo");
	return pUTI;
}

int allow_auth(char *login) {
	return allow_auth_default(login, "traff");
}

