#include "def.h"
#include "conf/conf.h"
#include "utils/utils.h"
#include "libkatrin-db-pgsql.h"

PGconn *conn;

void module_init(void) {
	dbConnect();
}

int dbConnect(void) {
	kcfg *cfg = get_katrin_cfg();

	if (cfg == NULL) {
		err("Can't get katrin configuration");
		return 1;
	}

	debug("Connect to DB...");

	char conninfo[MAX_QUERY_SIZE];
	if (sprintf(conninfo, "host=%s user=%s password=%s dbname=%s", 
		cfg->db_server, cfg->db_login, cfg->db_password, cfg->db_database) < 0) {
		err("sprintf error in dbConnect");
	}

	conn = PQconnectdb(conninfo);

	if (PQstatus(conn) != CONNECTION_OK) {
		err(" Error connection to DB: %s", PQerrorMessage(conn));
		PQfinish(conn);
		free_katrin_cfg(cfg);
		return 2;
    }

	debug(" Good connect to DB");
	free_katrin_cfg(cfg);
	return 0;
}

void dbDisconnect(void) {
	PQfinish(conn);
}

struct tariff *get_tariff(int id) {
	debug("get_tariff");
	char query[MAX_QUERY_SIZE];
	sprintf(query, "SELECT id, week_days FROM tariffs WHERE id='%d';", id);
	struct tariff *p = NULL;

	PGresult *res = katrin_pgsql_query(query);
	if (res == NULL)
		return NULL;

	if (PQntuples(res)==0) {
		warn("Can't get tariff!");
		return NULL;
	}
	p = Tariff(atoi(PQgetvalue(res, 0, 0)), PQgetvalue(res, 0, 3));

	PQclear(res);

	return p;
}

/* Additional specific function for pgsql db */

/* Exec sql query and return pointer to PGresult or NULL on error. 
 * @param query: pointer to query string 
 * return: pointer to PGresult or NULL on error */
static PGresult *katrin_pgsql_query(const char *query) {
	debug("katrin_pgsql_query");
	PGresult *res;

	while (PQstatus(conn) == CONNECTION_BAD) {
		debug("attempt connect to DB");
		PQreset(conn);
		usleep ( 1000000 ) ;
	}

	res = PQexec(conn, query);
	if (PQresultStatus(res) != PGRES_COMMAND_OK) {
		debug("Query: %s", query);
		err("Failed query: %s", PQerrorMessage(conn));
		PQclear(res);
		return NULL;
    }
	return res;
}

