#include <unistd.h>
#include <stdio.h>
#include <signal.h>

#include "def.h"
#include "drop/libkatrin-drop.h"

#include "utils/utils.h"
#include "utils/initial_daemon.h"
#include "conf/conf.h"
#include "module/module_api.h"

#define PIDFILE "/var/run/katrin/katrin-dropd.pid"

static void term_handler(int i) {
	debug("Kill katrin-dropd");
	if (unlink(PIDFILE)!=0)
		err("Can't unlink pid file %s", PIDFILE);
	closelog();
	debug("Terminating");
	exit(EXIT_SUCCESS);
}

/* Call it periodically to have account statuses up to date
 * and access for abonents enabled or disabled
 */
static void check_clients_access(void) {
	debug("check_abonents_access");
	struct abonentslist *al = NULL;
	
	al = db.db_api.update_account_state(Blocked);
	set_abonents_access(al, Blocked);
	free_abonentslist(al);

	al = db.db_api.update_account_state(Activated);
	set_abonents_access(al, Activated);
	free_abonentslist(al);
}

int main(int argc, char **argv) {
	if ((argc == 2 && strcmp(argv[1],"-d") != 0) || argc > 2) {
		printf("Usage katrind for daemon or katrin-dropd -d for output debug information\n");
		exit(EXIT_FAILURE);
	}

	if (argc == 1)
		daemonize();

	privs_client();
	write_pid(PIDFILE);

	/* Setting signal handlers map */
	struct sigaction sa;
	sa.sa_handler = term_handler;
	sigaction(SIGTERM, &sa, 0);

	kcfg *cfg = get_katrin_cfg();
	if (cfg == NULL) {
		return EXIT_FAILURE;
	}
	add_module(DB, cfg->db_mod);

	while(1) {
		check_clients_access();
		sleep(cfg->time_check_drop);
	}


	free_katrin_cfg(cfg);

	return EXIT_SUCCESS;
}


