#define _GNU_SOURCE
#include "libkatrin-drop.h"

/* Standart function for drop. Other special function placed in
 * drop program/scripts <type>-drop. Example ppp-drop
 **/

/* The dropclient function control how clients will drop with depend by clients.type
 **/
int set_abonents_access(struct abonentslist *al, enum accountState state) {
	debug("set_abonents_access with state %d",state);
	int ret=0;
	struct abonentslist *p = NULL;
	
	for(p=al;p!=NULL;p=p->next) {
		if(p->type == Client) {
			debug("Abonent is client");
			ret+=set_client_access(&p->abonent->client, &p->service, state);
		}
		else {
			debug("Abonent is clientgroup");
			ret+=set_clientgroup_access(&p->abonent->clientgroup, &p->service, state);
		}
		debug("ret is %d",ret);
	}

	return ret;
}

int set_client_access(struct client *c, struct service *s, enum accountState state) {
	struct paramlist *params = NULL;
	int ret = 0;
	if(!c->uparams)
		c->uparams = db.db_api.get_client_params(c->id, s->name);
	add_param(&c->uparams,"abonent", "client");
	if (state == Activated) 
		ret = do_action(Client, c->uparams, s, Enable);
	else
		ret = do_action(Client, c->uparams, s, Drop);

	return ret;
}

int set_clientgroup_access(struct clientgroup *g, struct service *s, enum accountState state) {
	struct paramlist *params = NULL;
	struct clientslist *members = NULL;
	int ret = 0;
	// Group specific actions
	if(!g->gparams)
		g->gparams = db.db_api.get_clientgroup_params(g->id, s->name);
	add_param(&g->gparams,"abonent", "clientgroup");
	if (state == Activated) 
		ret = do_action(ClientGroup, g->gparams, s, Enable);
	else
		ret = do_action(ClientGroup, g->gparams, s, Drop);

	if(!g->members) {
		debug("I'm trying to get clientgroup members");
		g->members = db.db_api.get_clientgroup_members(g->id);
	}

	// Specific actions for each client in group
	for(members = g->members;members!=NULL;members = members->next) {
		// pass group_action param to client script
		//add_param(&members->client->uparams,"group_action", "true");
		ret+=set_client_access(members->client, s, state);
	}

	return ret;
}

static int do_action(enum abonentType atype, struct paramlist *params, struct service *s, enum actionType action) {
	char *prog;
	// to build keyvalue list + \0
	char kvlist[(MAX_PARAM_NAME_LENGTH+MAX_PARAM_VALUE_LENGTH+4)*MAX_NUMBER_OF_PARAMS+1];
	char action_str[10];
	char second_part[MAX_PARAM_VALUE_LENGTH];
	char *typeAccess = NULL;
	int ret = 0;

	switch(action) {
		case Drop: strcpy(action_str, "drop");break;
		case Enable: strcpy(action_str, "enable");break;
		default:
			err("Unknown action %d",action);
			return 1;
	}

	debug("do action %s", action_str);

	if (atype == Client) {
		typeAccess = get_param_value("access_type", params);
		if (NULL == typeAccess) {
			err("Can not get typeAccess");
			return 4;
		}
		strcpy(second_part, typeAccess);
	}
	else {
		strcpy(second_part, "clientgroup");
	}

	paramlist2keyvaluelist(params, kvlist);
	
	ret = asprintf(&prog, "sudo %s/%s-%s-%s %s", BIN_DIR, s->name, second_part, action_str, kvlist);
	if (ret == -1) {
		err("asprintf: can't write name of drop program to string!");
		return 2;
	}
	/* exec <ServiceName>-<typeAccess|clientgroup>-<action> <kvlist> */
	debug("exec prog: %s", prog);
	ret = system(prog); 
	if ( ret == -1) {
		err("Can't execute program: %s", prog);
		return 3;
	}
	debug("program returned %d",ret);

	free(prog);
	return ret;
}
