#define _GNU_SOURCE
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <syslog.h>

#include "def.h"
#include "utils/utils.h"
#include "utils/initial_daemon.h"
#include "conf/conf.h"
#include "module/module_api.h"

#define PIDFILE "/var/run/katrin/katrind.pid"
#define PIPES_BUFFER_SIZE 20480


static void term_handler(int i) {
//	Kill all info modules
	debug("Kill all info modules");
	remove_all_info_modules();
	if (unlink(PIDFILE)!=0)
		err("Can't unlink pid file %s", PIDFILE);
	closelog();
	debug("Terminating");
	exit(EXIT_SUCCESS);
}


int main(int argc, char **argv) {

	if ((argc == 2 && strcmp(argv[1],"-d") != 0) || argc > 2) {
		printf("Usage katrind for daemon or katrind -d for output debug information\n");
		exit(EXIT_FAILURE);
	}

	if (argc == 1)
		daemonize();

	privs_client();

	write_pid(PIDFILE);

	/* Setting signal handlers map */
	struct sigaction sa;
	sa.sa_handler = term_handler;
	sigaction(SIGTERM, &sa, 0);

	debug("katrind pid is %d", getpid());

	/* Get initial configuration */
	kcfg *cfg = get_katrin_cfg();

	if (cfg == NULL) {
		err("Can't read config");
		return EXIT_FAILURE;
	}

	/* Startup */
	info("Start Katrin daemon");
	add_module(DB, cfg->db_mod);

	/*  Read service info from info collectors  */
	int pipedes[2];
	if (pipe(pipedes) == -1) {
		err("Can't open pipes: %s", strerror(errno));
		exit(EXIT_FAILURE);
	}

	char *info_mod = NULL;
	service_list *p = NULL;

	if (cfg->service_list == NULL) {
		err("Can't find info modules in config");
		exit(EXIT_FAILURE);
	}

	/* Loading business modules for each service */
	for (p = cfg->service_list ; p != NULL ; p = p->next) {
		debug("Service name: %s", p->name);
		add_module(BZ, p->name);

		list *l = NULL;

		for (l = p->info_mod; l != NULL ; l = l->next) {
			if (-1 == asprintf(&info_mod, "%s-%s", p->name, l->value)) {
				err("Can't allocate memory for asprintf");
				exit(EXIT_FAILURE);
			}
			debug("Info module: %s", info_mod);

			add_module(INFO, info_mod, pipedes);

			free(info_mod);
		}
	}

	close(pipedes[1]);

	char buf[PIPES_BUFFER_SIZE];
	int len = 0;
	int i = 0;
	struct service_info *si;
	/* Main collecting cycle */
	debug("Buffer size for read service info: %d bytes", PIPES_BUFFER_SIZE);
	while ((len = read(pipedes[0], buf, PIPES_BUFFER_SIZE)) != 0) {
		debug("%d bytes received", len);
		si = (struct service_info *)buf;
		for (i = 0; i <  len && si->len > 0; i += si->len) {
			si = (struct service_info *)&buf[i];
			debug("Service info was received: %s, si->len=%d, start byte=%d", si->type, si->len, i);
			if (get_service_cfg(cfg, si->type) == NULL)
				continue;
			/* Send service info to libkatrin-bz-<type_service> */
			bz_module *bz_mod =  get_bz_module(si->type);
			struct client_service_info *uService = bz_mod->bz_api.service2clientService(si);
			debug("cost: %1.5f", uService->cost);
			if (uService->cost > 0)
				db.db_api.writeoffcost(uService->cost, uService->account_id);
			free(uService);
		}

		if (si->len <= 0) {
			warn("si->len = %d; VARS: si->type = %s; len = %d; i=%d", si->len, si->type, len, i);
			warn("buf=%s", buf);
		}
	}
	err("Can't read pipes: %s", strerror(errno));
	close(pipedes[0]);	
	free_katrin_cfg(cfg);
	debug("Katrin exited");
	return 0;
}


