#define _GNU_SOURCE
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>

#include <sys/types.h>
#include <unistd.h>
#include <signal.h>

#include "utils/utils.h"

#include "module_api.h"
#include "module_api_declaration.h"

static int db_module_count = 0;
static int bz_module_count = 0;
static int info_module_count = 0;

static info_module info_module_array[INFO_MMAX];
static bz_module bz_module_array[BZ_MMAX];
db_module db;

int add_module(enum module_type type, const char *name, ...){
	debug("add_module");
	va_list args;
	va_start(args, name);

	int res;
	int *pipedes;

	if (name == NULL) {
		err("Module name pointer is NULL!");
		return -1;
	}

	if (strlen(name) >= NAME_LENGTH) {
		err("Name is too long to be a module name!");
		return -1;
	}

	switch (type) {
		case DB:
			res = add_db_module(name);
			break;

		case BZ:
			res = add_bz_module(name);
			break;

		case INFO:
			pipedes = va_arg(args, int *);
			res = add_info_module(name, pipedes);
			break;

		default:
			err("No such module type!");
			return -1;
	}

	va_end(args);

	return res;
}

static int add_db_module(const char* name) {
	if(db_module_count >= DB_MMAX){
		debug("DB already load");
		return 1;
	}

	if(!strncmp(name, db.name, strlen(name))){
		err("Error - module is already loaded");
		return 2;
	}

	memset(&db, 0, sizeof(db_module));

	int ret;

	ret = load_library("db", name, &(db.handleLibDB));
	if(ret){
		return 3;
	}

	strncpy(db.name,name, NAME_LENGTH);

	db.db_api.module_init = katrin_dlsym(db.handleLibDB,"module_init");
	db.db_api.dbDisconnect = katrin_dlsym(db.handleLibDB,"dbDisconnect");
	db.db_api.dbConnect = katrin_dlsym(db.handleLibDB,"dbConnect");
	db.db_api.writeoffcost = katrin_dlsym(db.handleLibDB,"writeoffcost");
	db.db_api.set_account_blocked = katrin_dlsym(db.handleLibDB,"set_account_blocked");
	db.db_api.checkloginpassword = katrin_dlsym(db.handleLibDB,"checkloginpassword");
	db.db_api.get_client = katrin_dlsym(db.handleLibDB,"get_client");
	db.db_api.get_tariff = katrin_dlsym(db.handleLibDB,"get_tariff");
	db.db_api.get_account = katrin_dlsym(db.handleLibDB,"get_account");
	db.db_api.get_client_services = katrin_dlsym(db.handleLibDB,"get_client_services");
	db.db_api.get_client_clientgroups_services = katrin_dlsym(db.handleLibDB,"get_client_clientgroups_services");
	db.db_api.update_account_state = katrin_dlsym(db.handleLibDB,"update_account_state");
	db.db_api.get_clientgroup_members = katrin_dlsym(db.handleLibDB,"get_clientgroup_members");
	db.db_api.addstat = katrin_dlsym(db.handleLibDB,"addstat");
	db.db_api.tariffid2filters = katrin_dlsym(db.handleLibDB,"tariffid2filters");
	db.db_api.tariffid2clients = katrin_dlsym(db.handleLibDB,"tariffid2clients");
	db.db_api.get_client_params = katrin_dlsym(db.handleLibDB,"get_client_params");
	db.db_api.get_clientgroup_params = katrin_dlsym(db.handleLibDB,"get_clientgroup_params");

	if(	db.db_api.module_init &&
		db.db_api.dbDisconnect &&
		db.db_api.dbConnect &&
		db.db_api.writeoffcost &&
		db.db_api.set_account_blocked &&
		db.db_api.checkloginpassword &&
		db.db_api.get_client &&
		db.db_api.get_tariff &&
		db.db_api.get_account &&
		db.db_api.get_client_services &&
		db.db_api.get_client_clientgroups_services &&
		db.db_api.update_account_state &&
		db.db_api.get_clientgroup_members &&
		db.db_api.addstat &&
		db.db_api.tariffid2filters &&
		db.db_api.tariffid2clients &&
		db.db_api.get_client_params &&
		db.db_api.get_clientgroup_params) {

		db.db_api.module_init();
		db_module_count++;
	} else {
		err("Not all db api functions were defined in %s db lib", name);
		dlclose(db.handleLibDB);
		return -2;
	}

	return 0;
}

static int add_bz_module(const char* name){
	debug("add_bz_module(%s)", name);
	int ret;

	if(bz_module_count >= BZ_MMAX){
		err("No enough space in the array of bz modules");
		return 1;
	}

	int i = 0;
	for(i = 0; i < bz_module_count; i++) {
		if(strncmp(name, bz_module_array[i].name, strlen(name)) == 0){
			err("Module is already loaded - exit!");
			return 2;
		}
	}

	memset(&bz_module_array[bz_module_count], 0, sizeof(bz_module));

	ret = load_library("bz", name, &(bz_module_array[bz_module_count].handleLibBZ));
	if (ret) {
		return 3;
	}

	strncpy(bz_module_array[bz_module_count].name,name, NAME_LENGTH);

	bz_module_array[bz_module_count].bz_api.service2clientService =
		katrin_dlsym(bz_module_array[bz_module_count].handleLibBZ,"service2clientService");

	bz_module_array[bz_module_count].bz_api.allow_auth =
		katrin_dlsym(bz_module_array[bz_module_count].handleLibBZ, "allow_auth");

	if(bz_module_array[bz_module_count].bz_api.service2clientService &&
		bz_module_array[bz_module_count].bz_api.allow_auth) {

		bz_module_count++;
	} else {
		err("No all bz api funtions were define in %s bz lib", name);
		dlclose(bz_module_array[bz_module_count].handleLibBZ);
		return -1;
	}

	return 0;
}

static int add_info_module(const char* name, int *pipedes) {
	if(info_module_count >= INFO_MMAX) {
		err("No enough space in the array of info modules");
		return 1;
	}

	int i = 0;
	for(i = 0; i < info_module_count; i++) {
		if(strncmp(name, info_module_array[i].name, strlen(name)) == 0){
			err("Module is already loaded - exit!");
			return 2;
		}
	}

	memset(&info_module_array[info_module_count], 0, sizeof(info_module));

	strncpy(info_module_array[info_module_count].name,name, NAME_LENGTH);

	char *info_full_name;
	if ( -1 == asprintf(&info_full_name, "%s/katrin-info-%s", BIN_DIR, name)) {
		err("Can't allocate memory in asprintf");
		exit(EXIT_FAILURE);
	}

	debug("Load info module: %s", info_full_name);

	pid_t pid;
	pid = fork();

	if ( pid == 0 ) {
		close(pipedes[0]);
		dup2(pipedes[1], 1);

		execve(info_full_name, NULL, NULL);
		close(pipedes[1]);
	}

	if(pid > 0) {
		info_module_array[info_module_count].pid = pid;
		info_module_count++;
	} else {
		err("Can't fork for exec info module %s", name);
		free(info_full_name);
		return -1;
	}
	free(info_full_name);
	return 0;
}

void remove_all_info_modules() {
	int i = 0;

	for (i = 0; i < info_module_count; i++) {
		debug("Kill %s info module", info_module_array[i].name);

		if (0 != kill(info_module_array[i].pid, SIGTERM))
			err("Can't kill %s info module", info_module_array[i].name);
	}
}

bz_module *get_bz_module(const char* name) {
	int i = 0;

	for (i = 0; i < bz_module_count; i++) {
		if (strncmp(bz_module_array[i].name, name, strlen(name)) == 0)
			return &bz_module_array[i];
	}

	return NULL;
}


static void *katrin_dlsym(void *handle, const char *symbol) {
	void *p = dlsym(handle, symbol);
	if(!p)
		err("Can't find handle for symbol %s",symbol);
	return p;
}
