#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <pwd.h>
#include <grp.h>
#include <fcntl.h>

#include "utils/utils.h"

#define KATRIN_USER "_katrin"
#define KATRIN_GROUP "_katrin"

void privs_client(void) {
	debug("privs_client");

	struct group *gr = NULL;
	gr = getgrnam(KATRIN_GROUP);
	if (gr == NULL) {
		err("Can't find group: %s", KATRIN_GROUP);
		exit(EXIT_FAILURE);
	}
	debug("Set gid: %d", gr->gr_gid);
	if (setgid(gr->gr_gid) != 0)
		err("failed setgid(): %s", strerror(errno));


	struct passwd *pw = NULL;
	pw = getpwnam(KATRIN_USER);
	if (pw == NULL) {
		err("Can't find client: %s", KATRIN_USER);
		exit(EXIT_FAILURE);
	}
	debug("Set uid: %d", pw->pw_uid);
	if (setuid(pw->pw_uid)!=0)
		err("failed setuid(): %s", strerror(errno));

}

void write_pid(const char *pidfile) {
	debug("write_pid");
	FILE *pidf = NULL;
	pidf = fopen(pidfile, "w");

	if (pidf == NULL) {
		err("error opening PidFile '%s': %s", pidfile, strerror(errno));
		exit(EXIT_FAILURE);
	}

	fprintf(pidf, "%lu\n", (unsigned long) getpid());
	fclose(pidf);
}

void daemonize(void) {
	debug("Daemonizing...");

	switch (fork()) {
	case -1:
		perror("fork");
		exit(EXIT_FAILURE);

	case 0:
		break;

	default:
		exit(EXIT_SUCCESS);
	}

	setsid();

	int sd = 0;
	if ((sd = open("/dev/null", O_RDWR)) == -1 &&
		(sd = open("/dev/null", O_RDONLY)) == -1 &&
		/* Panic -- open *something* */
		(sd = open( "/", O_RDONLY)) == -1 ) {
		perror("/dev/null");
	} else {
		/* redirect stdin, stdout, stderr to /dev/null */
		dup2(sd, fileno(stdin));
		dup2(sd, fileno(stdout));
		dup2(sd, fileno(stderr));
	}


	if(getppid() != 1) {
		signal(SIGTTOU, SIG_IGN);
		signal(SIGTTIN, SIG_IGN);
		signal(SIGTSTP, SIG_IGN);
	}

	return 0;

}

